from __future__ import unicode_literals

import django

from django.conf import settings
from django.db.models import query, OneToOneRel
from django.db import models, transaction
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.contenttypes.models import ContentType
try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.auth.models import Group, Permission
from django.utils import timezone
import logging
from softdelete.signals import *

try:
    USE_SOFTDELETE_GROUP = settings.USE_SOFTDELETE_GROUP
except:
    USE_SOFTDELETE_GROUP = False


def _determine_change_set(obj, create=True):
    try:
        qs = SoftDeleteRecord.objects.filter(content_type=ContentType.objects.get_for_model(obj),
                                             object_id=str(obj.pk)).latest('created_date').changeset
        logging.debug("Found changeset via latest recordset")
    except:
        try:
            qs = ChangeSet.objects.filter(content_type=ContentType.objects.get_for_model(obj),
                                          object_id=str(obj.pk)).latest('created_date')
            logging.debug("Found changeset")
        except:
            if create:
                qs = ChangeSet.objects.create(content_type=ContentType.objects.get_for_model(obj),
                                              object_id=str(obj.pk))
                logging.debug("Creating changeset")
            else:
                logging.debug("Raising ObjectDoesNotExist")
                raise ObjectDoesNotExist
    return qs


class SoftDeleteQuerySet(query.QuerySet):
    def all_with_deleted(self):
        qs = super(SoftDeleteQuerySet, self).all()
        qs.__class__ = SoftDeleteQuerySet
        return qs

    def delete(self, using='default', *args, **kwargs):
        if not len(self):
            return
        cs = kwargs.get('changeset')
        logging.debug("STARTING QUERYSET SOFT-DELETE: %s. %s", self, len(self))
        for obj in self:
            rs, c = SoftDeleteRecord.objects.get_or_create(changeset=cs or _determine_change_set(obj),
                                                           content_type=ContentType.objects.get_for_model(obj),
                                                           object_id=str(obj.pk))
            logging.debug(" -----  CALLING delete() on %s", obj)
            obj.delete(using, *args, **kwargs)

    def undelete(self, using='default', *args, **kwargs):
        logging.debug("UNDELETING %s", self)
        for obj in self:
            cs = _determine_change_set(obj)
            cs.undelete()
        logging.debug("FINISHED UNDELETING %s", self)


class SoftDeleteManager(models.Manager):

    def _get_base_queryset(self):
        '''
        Convenience method for grabbing the base query set. Accounts for the
        deprecation of get_query_set in Django 18.
        '''

        if django.VERSION >= (1, 8, 0, 'final', 0):
            return super(SoftDeleteManager, self).get_queryset()
        else:
            return super(SoftDeleteManager, self).get_query_set()

    def _get_self_queryset(self):
        '''
        Convenience method for grabbing the query set. Accounts for the
        deprecation of get_query_set in Django 1.8
        '''

        if django.VERSION >= (1, 8, 0, 'final', 0):
            return self.get_queryset()
        else:
            return self.get_query_set()

    def get_query_set(self):
        qs = super(SoftDeleteManager, self).get_query_set().filter(
            deleted_at=0)
        if not issubclass(qs.__class__, SoftDeleteQuerySet):
            qs.__class__ = SoftDeleteQuerySet
        return qs

    def get_queryset(self):
        qs = super(SoftDeleteManager, self).get_queryset().filter(
            deleted_at=0)
        if not issubclass(qs.__class__, SoftDeleteQuerySet):
            qs.__class__ = SoftDeleteQuerySet
        return qs

    def all_with_deleted(self, prt=False):
        if hasattr(self, 'core_filters'):  # it's a RelatedManager
            qs = self._get_base_queryset().filter(**self.core_filters)
        else:
            qs = self._get_base_queryset()
        if not issubclass(qs.__class__, SoftDeleteQuerySet):
            qs.__class__ = SoftDeleteQuerySet
        return qs

    def deleted_set(self):
        qs = self._get_base_queryset().filter(deleted_at__gt=0)
        if not issubclass(qs.__class__, SoftDeleteQuerySet):
            qs.__class__ = SoftDeleteQuerySet
        return qs

    def get(self, *args, **kwargs):
        if 'pk' in kwargs:
            return self.all_with_deleted().get(*args, **kwargs)
        else:
            return self._get_self_queryset().get(*args, **kwargs)

    def filter(self, *args, **kwargs):
        if 'pk' in kwargs:
            qs = self.all_with_deleted().filter(*args, **kwargs)
        else:
            qs = self._get_self_queryset().filter(*args, **kwargs)
        if not issubclass(qs.__class__, SoftDeleteQuerySet):
            qs.__class__ = SoftDeleteQuerySet
        return qs


class SoftDeleteObject(models.Model):
    deleted_at = models.BigIntegerField(
        default=0, editable=False, db_index=True
    )
    objects = SoftDeleteManager()

    class Meta:
        abstract = True
        permissions = (
            ('can_undelete', 'Can undelete this object'),
            )

    def __init__(self, *args, **kwargs):
        super(SoftDeleteObject, self).__init__(*args, **kwargs)
        self.__dirty = False

    def get_deleted(self):
        return self.deleted_at != 0

    def set_deleted(self, d):
        """Called via the admin interface (if user checks the "deleted" checkox)"""
        if d and not self.deleted_at:
            self.__dirty = True
            self.deleted_at = int(timezone.now().timestamp() * 1000)
        elif not d and self.deleted_at:
            self.__dirty = True
            self.deleted_at = 0

    deleted = property(get_deleted, set_deleted)

    def _do_delete(self, changeset, related):
        rel = related.get_accessor_name()

        # Sometimes there is nothing to delete
        if not hasattr(self, rel):
            return

        try:
            if related.one_to_one:
                getattr(self, rel).delete(changeset=changeset)
            else:
                getattr(self, rel).all().delete(changeset=changeset)
        except:
            try:
                if related.one_to_one:
                    getattr(self, rel).delete()
                else:
                    getattr(self, rel).all().delete()
            except Exception as e:
                if getattr(settings, "SOFTDELETE_CASCADE_ALLOW_DELETE_ALL", True):
                    # fallback to delete all objects in the related field's model class
                    # to maintain previous behaviour (before setting was added)
                    try:
                        getattr(self, rel).__class__.objects.all().delete(
                            changeset=changeset)
                    except:
                        getattr(self, rel).__class__.objects.all().delete()
                else:
                    raise e

    @transaction.atomic
    def hard_delete(self, *args, **kwargs):
        super(SoftDeleteObject, self).delete(*args, **kwargs)

    def delete(self, *args, **kwargs):
        if self.deleted_at:
            logging.debug("HARD DELETEING type %s, %s", type(self), self)
            try:
                cs = ChangeSet.objects.get(
                    content_type=ContentType.objects.get_for_model(self),
                    object_id=self.pk)
                cs.delete()
                super(SoftDeleteObject, self).delete(*args, **kwargs)
            except:
                try:
                    cs = kwargs.get('changeset') or _determine_change_set(self)
                    rs = SoftDeleteRecord.objects.get(
                        changeset=cs,
                        content_type=ContentType.objects.get_for_model(self),
                        object_id=self.pk)
                    if rs.changeset.soft_delete_records.count() == 1:
                        cs.delete()
                    else:
                        rs.delete()
                    super(SoftDeleteObject, self).delete(*args, **kwargs)
                except:
                    pass
        else:
            using = kwargs.get('using', settings.DATABASES['default'])
            models.signals.pre_delete.send(sender=self.__class__,
                                           instance=self,
                                           using=using)
            pre_soft_delete.send(sender=self.__class__,
                                 instance=self,
                                 using=using)
            logging.debug('SOFT DELETING type: %s, %s', type(self), self)
            cs = kwargs.get('changeset') or _determine_change_set(self)
            SoftDeleteRecord.objects.get_or_create(
                changeset=cs,
                content_type=ContentType.objects.get_for_model(self),
                object_id=self.pk)
            self.deleted_at = int(timezone.now().timestamp() * 1000)
            self.save()
            all_related = [
                f for f in self._meta.get_fields()
                if (f.one_to_many or f.one_to_one)
                   and f.auto_created and not f.concrete
            ]

            for x in all_related:
                if x.on_delete.__name__ not in ['DO_NOTHING', 'SET_NULL']:
                    self._do_delete(cs, x)
                if x.on_delete.__name__ == 'SET_NULL':
                    related_name = x.get_accessor_name()
                    if isinstance(x, OneToOneRel):
                        if getattr(self, related_name, None) is None:
                            continue
                        related = getattr(self, related_name)
                        if isinstance(related, models.Model):
                            setattr(related, x.remote_field.name, None)
                            related.save(update_fields=[x.remote_field.name])
                    else:
                        getattr(self, related_name).all().update(**{x.remote_field.name: None})
            logging.debug("FINISHED SOFT DELETING RELATED %s", self)
            models.signals.post_delete.send(sender=self.__class__,
                                            instance=self,
                                            using=using)
            post_soft_delete.send(sender=self.__class__,
                                  instance=self,
                                  using=using)

    def _do_undelete(self, using='default'):
        pre_undelete.send(sender=self.__class__,
                          instance=self,
                          using=using)
        self.deleted_at = 0
        self.save()
        post_undelete.send(sender=self.__class__,
                           instance=self,
                           using=using)

    def undelete(self, using='default', *args, **kwargs):
        logging.debug('UNDELETING %s' % self)
        cs = kwargs.get('changeset') or _determine_change_set(self, False)
        cs.undelete(using)
        logging.debug('FINISHED UNDELETING RELATED %s', self)

    def save(self, **kwargs):
        super(SoftDeleteObject, self).save(**kwargs)
        if self.__dirty:
            self.__dirty = False
            if not self.deleted:
                self.undelete()
            else:
                self.delete()

class ChangeSet(models.Model):
    id = models.BigAutoField(
        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
    )
    created_date = models.DateTimeField(default=timezone.now)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.CharField(max_length=100)
    record = GenericForeignKey('content_type', 'object_id')

    class Meta:
        index_together = [
            ("content_type", "object_id"),
        ]

    def get_content(self):
        self.record = self.content_type.model_class().objects.get(
            pk=self.object_id)
        return self.record

    def set_content(self, obj):
        self.record = obj

    def undelete(self, using='default'):
        logging.debug("CHANGESET UNDELETE: %s" % self)
        self.content._do_undelete(using)
        for related in self.soft_delete_records.all():
            related.undelete(using)
        self.delete()
        logging.debug("FINISHED CHANGESET UNDELETE: %s", self)

    def __str__(self):
        return 'Changeset: %s, %s' % (self.created_date, self.record)

    content = property(get_content, set_content)

class SoftDeleteRecord(models.Model):
    id = models.BigAutoField(
        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
    )
    changeset = models.ForeignKey(
        ChangeSet,
        related_name='soft_delete_records',
        on_delete=models.CASCADE
    )
    created_date = models.DateTimeField(default=timezone.now)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.CharField(max_length=100)
    record = GenericForeignKey('content_type', 'object_id')

    class Meta:
        unique_together = (('changeset', 'content_type', 'object_id'),)
        index_together = [
            ("content_type", "object_id"),
        ]

    def get_content(self):
        self.record = self.content_type.model_class().objects.get(pk=self.object_id)
        return self.record

    def set_content(self, obj):
        self.record = obj

    def undelete(self, using='default'):
        self.content._do_undelete(using)

    def __str__(self):
        return u'SoftDeleteRecord: (%s), (%s/%s), %s' % (
            self.content,
            self.content_type,
            self.object_id,
            self.changeset.created_date)

    content = property(get_content, set_content)


def assign_permissions(user_or_group):
    for model in ['ChangeSet', 'SoftDeleteRecord']:
        ct = ContentType.objects.get(app_label="softdelete",
                                     model=model.lower())
        p, pc = Permission.objects.get_or_create(
            name="Can undelete a soft-deleted object",
            codename="can_undelete",
            content_type=ct)
        permissions = [p]
        for permission in ['add_%s' % model.lower(),
                           'change_%s' % model.lower(),
                           'delete_%s' % model.lower(),
                           'can_undelete']:
            for perm_obj in Permission.objects.filter(codename=permission):
                permissions.append(perm_obj)
        perm_list = getattr(user_or_group, 'permissions',
                            getattr(user_or_group, 'user_permissions'))
        [perm_list.add(x) for x in permissions]
        user_or_group.save()
    return user_or_group


def create_group():
    if USE_SOFTDELETE_GROUP:
        gr, cr = Group.objects.get_or_create(name='Softdelete User')
        if cr:
            assign_permissions(gr)
        return gr



from django.urls import include, re_path
from softdelete.views import *

urlpatterns = [
    re_path(r'^changeset/(?P<changeset_pk>\d+?)/undelete/$',
        ChangeSetUpdate.as_view(),
        name="softdelete.changeset.undelete"),
    re_path(r'^changeset/(?P<changeset_pk>\d+?)/$',
        ChangeSetDetail.as_view(),
        name="softdelete.changeset.view"),
    re_path(r'^changeset/$',
        ChangeSetList.as_view(),
        name="softdelete.changeset.list"),
]

import sys
if 'test' in sys.argv:
    import django
    from django.contrib import admin
    admin.autodiscover()

    if django.VERSION[0] >= 2:
        from django.urls import path
        urlpatterns.append(path('admin/', admin.site.urls))
        urlpatterns.append(path('accounts/', include('django.contrib.auth.urls')))
    else:
        urlpatterns.append(re_path(r'^admin/', include(admin.site.urls)))
        urlpatterns.append(re_path(r'^accounts/', include('django.contrib.auth.urls')))



from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase, Client, override_settings
from django.contrib.auth.models import User
from django.db import models
from softdelete.test_softdelete_app.models import (
    TestModelOne,
    TestModelTwoCascade,
    TestModelThree,
    TestModelThrough,
    TestModelTwoDoNothing,
    TestModelTwoSetNull,
    TestModelTwoSetNullOneToOne,
    TestModelO2OFemaleSetNull,
    TestModelBaseO2OMale,
    TestModelO2OFemaleCascade,
    TestModelO2OFemaleCascadeNoSD,
    TestModelO2OFemaleCascadeErrorOnDelete,
)
from softdelete.tests.constanats import TEST_MODEL_ONE_COUNT, TEST_MODEL_TWO_TOTAL_COUNT, TEST_MODEL_THREE_COUNT, \
    TEST_MODEL_TWO_LIST, TEST_MODEL_TWO_CASCADE_COUNT, TEST_MODEL_TWO_SET_NULL_COUNT, TEST_MODEL_TWO_DO_NOTHING_COUNT
from softdelete.models import *
from softdelete.signals import *
import logging

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse


class BaseTest(TestCase):
    def setUp(self):
        # update TEST_MODEL_ONE_COUNT constant if you initialize more or less instance of TestModelOne
        self.tmo1 = TestModelOne.objects.create(extra_bool=True)
        self.tmo2 = TestModelOne.objects.create(extra_bool=False)

        for x in range(TEST_MODEL_TWO_TOTAL_COUNT):
            TEST_MODEL_TWO_LIST[x % len(TEST_MODEL_TWO_LIST)] \
                .objects.create(extra_int=x, tmo=self.tmo1 if x % TEST_MODEL_ONE_COUNT else self.tmo2)
        for x in range(TEST_MODEL_TWO_TOTAL_COUNT):
            if x % TEST_MODEL_ONE_COUNT:
                left_side = self.tmo1
            else:
                left_side = self.tmo2
            for x in range(TEST_MODEL_TWO_TOTAL_COUNT):
                t3 = TestModelThree.objects.create()
                tmt = TestModelThrough.objects.create(tmo1=left_side, tmo3=t3)
        self.user = User.objects.create_user(username='SoftdeleteUser',
                                             password='SoftdeletePassword',
                                             email='softdeleteuser@example.com')
        gr = create_group()
        if USE_SOFTDELETE_GROUP:
            gr = Group.objects.get(name="Softdelete User")
            self.user.groups.add(gr)
            self.user.save()
            gr.save()
        else:
            assign_permissions(self.user)
        self.user.save()
        self.unauthorized = User.objects.create_user(username='NonSoftdeleteUser',
                                                     password='NonSoftdeletePassword',
                                                     email='nonsoftdeleteuser@example.com')


class InitialTest(BaseTest):

    def test_simple_delete(self):
        self.assertEquals(TEST_MODEL_ONE_COUNT, TestModelOne.objects.count())

        self.assertEquals(TEST_MODEL_ONE_COUNT, TestModelOne.objects.all_with_deleted().count())

        self.assertEquals(TEST_MODEL_TWO_CASCADE_COUNT, TestModelTwoCascade.objects.count())
        self.assertEquals(TEST_MODEL_TWO_SET_NULL_COUNT, TestModelTwoSetNull.objects.count())
        self.assertEquals(TEST_MODEL_TWO_DO_NOTHING_COUNT, TestModelTwoDoNothing.objects.count())

        self.assertEquals(TEST_MODEL_TWO_CASCADE_COUNT, TestModelTwoCascade.objects.all_with_deleted().count())
        self.assertEquals(TEST_MODEL_TWO_DO_NOTHING_COUNT, TestModelTwoDoNothing.objects.all_with_deleted().count())
        self.assertEquals(TEST_MODEL_TWO_SET_NULL_COUNT, TestModelTwoSetNull.objects.all_with_deleted().count())

        self.assertEquals(TEST_MODEL_THREE_COUNT, TestModelThree.objects.count())

        self.assertEquals(TEST_MODEL_THREE_COUNT, TestModelThree.objects.all_with_deleted().count())

        self.tmo1.delete()

        self.assertEquals(TEST_MODEL_ONE_COUNT - 1, TestModelOne.objects.count())
        self.assertEquals(TEST_MODEL_ONE_COUNT, TestModelOne.objects.all_with_deleted().count())

        self.assertEquals(TEST_MODEL_TWO_CASCADE_COUNT - 2, TestModelTwoCascade.objects.count())
        self.assertEquals(TEST_MODEL_TWO_SET_NULL_COUNT, TestModelTwoSetNull.objects.count())
        self.assertEquals(TEST_MODEL_TWO_DO_NOTHING_COUNT, TestModelTwoDoNothing.objects.count())

        self.assertEquals(TEST_MODEL_TWO_CASCADE_COUNT, TestModelTwoCascade.objects.all_with_deleted().count())
        self.assertEquals(TEST_MODEL_TWO_SET_NULL_COUNT, TestModelTwoSetNull.objects.all_with_deleted().count())
        self.assertEquals(TEST_MODEL_TWO_DO_NOTHING_COUNT, TestModelTwoDoNothing.objects.all_with_deleted().count())

        self.assertEquals(TEST_MODEL_THREE_COUNT, TestModelThree.objects.count())
        self.assertEquals(TEST_MODEL_THREE_COUNT, TestModelThree.objects.all_with_deleted().count())

    def test_simple_multi_delete(self):
        self.assertEquals(TEST_MODEL_ONE_COUNT, TestModelOne.objects.count())

        self.assertEquals(TEST_MODEL_ONE_COUNT, TestModelOne.objects.all_with_deleted().count())

        self.assertEquals(TEST_MODEL_TWO_CASCADE_COUNT, TestModelTwoCascade.objects.count())
        self.assertEquals(TEST_MODEL_TWO_SET_NULL_COUNT, TestModelTwoSetNull.objects.count())
        self.assertEquals(TEST_MODEL_TWO_DO_NOTHING_COUNT, TestModelTwoDoNothing.objects.count())

        self.assertEquals(TEST_MODEL_TWO_CASCADE_COUNT, TestModelTwoCascade.objects.all_with_deleted().count())
        self.assertEquals(TEST_MODEL_TWO_DO_NOTHING_COUNT, TestModelTwoDoNothing.objects.all_with_deleted().count())
        self.assertEquals(TEST_MODEL_TWO_SET_NULL_COUNT, TestModelTwoSetNull.objects.all_with_deleted().count())

        self.assertEquals(TEST_MODEL_THREE_COUNT, TestModelThree.objects.count())

        self.assertEquals(TEST_MODEL_THREE_COUNT, TestModelThree.objects.all_with_deleted().count())

        self.tmo1.delete()
        self.tmo2.delete()

        self.assertEquals(TEST_MODEL_ONE_COUNT - 2, TestModelOne.objects.count())
        self.assertEquals(TEST_MODEL_ONE_COUNT, TestModelOne.objects.all_with_deleted().count())

        self.assertEquals(TEST_MODEL_TWO_CASCADE_COUNT - 4, TestModelTwoCascade.objects.count())
        self.assertEquals(TEST_MODEL_TWO_SET_NULL_COUNT, TestModelTwoSetNull.objects.count())
        self.assertEquals(TEST_MODEL_TWO_DO_NOTHING_COUNT, TestModelTwoDoNothing.objects.count())

        self.assertEquals(TEST_MODEL_TWO_CASCADE_COUNT, TestModelTwoCascade.objects.all_with_deleted().count())
        self.assertEquals(TEST_MODEL_TWO_SET_NULL_COUNT, TestModelTwoSetNull.objects.all_with_deleted().count())
        self.assertEquals(TEST_MODEL_TWO_DO_NOTHING_COUNT, TestModelTwoDoNothing.objects.all_with_deleted().count())

        self.assertEquals(TEST_MODEL_THREE_COUNT, TestModelThree.objects.count())
        self.assertEquals(TEST_MODEL_THREE_COUNT, TestModelThree.objects.all_with_deleted().count())


class DeleteTest(BaseTest):
    def pre_delete(self, *args, **kwargs):
        self.pre_delete_called = True

    def post_delete(self, *args, **kwargs):
        self.post_delete_called = True

    def pre_soft_delete(self, *args, **kwargs):
        self.pre_soft_delete_called = True

    def post_soft_delete(self, *args, **kwargs):
        self.post_soft_delete_called = True

    def _pretest(self):
        self.pre_delete_called = False
        self.post_delete_called = False
        self.pre_soft_delete_called = False
        self.post_soft_delete_called = False
        models.signals.pre_delete.connect(self.pre_delete)
        models.signals.post_delete.connect(self.post_delete)
        pre_soft_delete.connect(self.pre_soft_delete)
        post_soft_delete.connect(self.post_soft_delete)
        self.assertEquals(TEST_MODEL_ONE_COUNT, TestModelOne.objects.count())
        self.assertEquals(TEST_MODEL_TWO_CASCADE_COUNT, TestModelTwoCascade.objects.count())
        self.assertEquals(TEST_MODEL_TWO_SET_NULL_COUNT, TestModelTwoSetNull.objects.count())
        self.assertEquals(TEST_MODEL_TWO_DO_NOTHING_COUNT, TestModelTwoDoNothing.objects.count())
        self.assertEquals(TEST_MODEL_THREE_COUNT, TestModelThree.objects.count())
        self.assertFalse(self.tmo1.deleted)
        self.assertFalse(self.pre_delete_called)
        self.assertFalse(self.post_delete_called)
        self.assertFalse(self.pre_soft_delete_called)
        self.assertFalse(self.post_soft_delete_called)
        self.cs_count = ChangeSet.objects.count()
        self.rs_count = SoftDeleteRecord.objects.count()

    def _posttest(self):
        self.tmo1 = TestModelOne.objects.get(pk=self.tmo1.pk)
        self.tmo2 = TestModelOne.objects.get(pk=self.tmo2.pk)
        self.assertTrue(self.tmo1.deleted)
        self.assertFalse(self.tmo2.deleted)
        self.assertTrue(self.pre_delete_called)
        self.assertTrue(self.post_delete_called)
        self.assertTrue(self.pre_soft_delete_called)
        self.assertTrue(self.post_soft_delete_called)
        self.tmo1.undelete()

    def test_delete(self):
        self._pretest()
        self.tmo1.delete()
        self.assertEquals(self.cs_count + 1, ChangeSet.objects.count())
        self.assertEquals(self.rs_count + (TEST_MODEL_THREE_COUNT // 2  # half is assigned by tmo1
                                           + TEST_MODEL_TWO_CASCADE_COUNT // 2  # half assigned to cascade model two
                                           + 1  # tmo1 itself
                                           ), SoftDeleteRecord.objects.count())
        self._posttest()

    def test_hard_delete(self):
        self._pretest()
        tmo_tmp = TestModelOne.objects.create(extra_bool=True)
        tmo_tmp.delete()
        self.assertEquals(self.cs_count + 1, ChangeSet.objects.count())
        self.assertEquals(self.rs_count + 1, SoftDeleteRecord.objects.count())
        tmo_tmp.delete()
        self.assertEquals(self.cs_count, ChangeSet.objects.count())
        self.assertEquals(self.rs_count, SoftDeleteRecord.objects.count())
        self.assertRaises(TestModelOne.DoesNotExist,
                          TestModelOne.objects.get,
                          pk=tmo_tmp.pk)

    def test_filter_delete(self):
        self._pretest()
        TestModelOne.objects.filter(pk=1).delete()
        self.assertEquals(self.cs_count + 1, ChangeSet.objects.count())
        self.assertEquals(self.rs_count + (TEST_MODEL_THREE_COUNT // 2  # half is assigned by tmo1
                                           + TEST_MODEL_TWO_CASCADE_COUNT // 2  # half assigned to cascade model two
                                           + 1  # tmo1 itself
                                           ), SoftDeleteRecord.objects.count())
        self._posttest()

    def test_set_null_on_one_to_one(self):
        """
        Make sure reverse `OneToOne` fields are set to `None` upon soft delete.

        When an instance is soft deleted and other instances have a `OneToOne`
        relation to this instance with `on_delete=SET_NULL`, the other
        instances should have their relation set to `None`.
        """
        # Create two instances, one with a relation to the other.
        to_be_deleted = TestModelOne.objects.create()
        other_with_relation = TestModelTwoSetNullOneToOne.objects.create(
            tmo=to_be_deleted,
            extra_int=0,
        )

        # Make sure the relation is there before soft deleting.
        self.assertEqual(other_with_relation.tmo, to_be_deleted)

        # Then delete the instance and expect the relation from the other
        # instance is now `None`.
        to_be_deleted.delete()
        other_with_relation.refresh_from_db()
        self.assertIsNone(other_with_relation.tmo)



class AdminTest(BaseTest):
    def test_admin(self):
        client = Client()
        u = User.objects.create_user(username='test-user', password='test',
                                     email='test-user@example.com')
        u.is_staff = True
        u.is_superuser = True
        u.save()
        self.assertFalse(self.tmo1.deleted)
        client.login(username='test-user', password='test')
        url = '/admin/test_softdelete_app/testmodelone/1/'
        tmo = client.get(url)
        # the admin URLs changed with v1.9 change our expectation if it makes sense version wise.
        if tmo.status_code == 302 and tmo['Location'].endswith('change/') and (1, 9) <= django.VERSION:
            url = tmo['Location']
            tmo = client.get(url)
        self.assertEquals(tmo.status_code, 200)
        tmo = client.post(url, {'extra_bool': '1', 'deleted': '1'})
        self.assertEquals(tmo.status_code, 302)
        self.tmo1 = TestModelOne.objects.get(pk=self.tmo1.pk)
        self.assertTrue(self.tmo1.deleted)


class AuthorizationTest(BaseTest):
    def test_permission_needed(self):
        cl = Client()
        cl.login(username='NonSoftdeleteUser',
                 password='NonSoftdeletePassword')
        rv = cl.get(reverse('softdelete.changeset.list'))
        self.assertEquals(rv.status_code, 302)
        rv = cl.get(reverse('softdelete.changeset.view', args=(1,)))
        self.assertEquals(rv.status_code, 302)
        rv = cl.get(reverse('softdelete.changeset.undelete', args=(1,)))
        self.assertEquals(rv.status_code, 302)


class UndeleteTest(BaseTest):
    def pre_undelete(self, *args, **kwargs):
        self.pre_undelete_called = True

    def post_undelete(self, *args, **kwargs):
        self.post_undelete_called = True

    def test_undelete(self):
        self.pre_undelete_called = False
        self.post_undelete_called = False
        pre_undelete.connect(self.pre_undelete)
        post_undelete.connect(self.post_undelete)
        self.assertFalse(self.pre_undelete_called)
        self.assertFalse(self.post_undelete_called)
        self.cs_count = ChangeSet.objects.count()
        self.rs_count = SoftDeleteRecord.objects.count()
        self.tmo1.delete()
        self.assertEquals(self.cs_count + 1, ChangeSet.objects.count())
        self.assertEquals(self.rs_count + (TEST_MODEL_THREE_COUNT // 2  # half is assigned by tmo1
                                           + TEST_MODEL_TWO_CASCADE_COUNT // 2  # half assigned to cascade model two
                                           + 1  # tmo1 itself
                                           ), SoftDeleteRecord.objects.count())
        self.tmo1 = TestModelOne.objects.get(pk=self.tmo1.pk)
        self.tmo2 = TestModelOne.objects.get(pk=self.tmo2.pk)
        self.assertTrue(self.tmo1.deleted)
        self.assertFalse(self.tmo2.deleted)
        self.tmo1.undelete()
        self.assertEquals(0, ChangeSet.objects.count())
        self.assertEquals(0, SoftDeleteRecord.objects.count())
        self.tmo1 = TestModelOne.objects.get(pk=self.tmo1.pk)
        self.tmo2 = TestModelOne.objects.get(pk=self.tmo2.pk)
        self.assertFalse(self.tmo1.deleted)
        self.assertFalse(self.tmo2.deleted)
        self.assertTrue(self.pre_undelete_called)
        self.assertTrue(self.post_undelete_called)

        self.tmo1.delete()
        self.assertEquals(self.cs_count + 1, ChangeSet.objects.count())
        self.assertEquals(self.rs_count + (TEST_MODEL_THREE_COUNT // 2  # half is assigned by tmo1
                                           + TEST_MODEL_TWO_CASCADE_COUNT // 2  # half assigned to cascade model two
                                           + 1  # tmo1 itself
                                           ), SoftDeleteRecord.objects.count())
        self.tmo1 = TestModelOne.objects.get(pk=self.tmo1.pk)
        self.tmo2 = TestModelOne.objects.get(pk=self.tmo2.pk)
        self.assertTrue(self.tmo1.deleted)
        self.assertFalse(self.tmo2.deleted)
        TestModelOne.objects.deleted_set().undelete()
        self.assertEquals(0, ChangeSet.objects.count())
        self.assertEquals(0, SoftDeleteRecord.objects.count())
        self.tmo1 = TestModelOne.objects.get(pk=self.tmo1.pk)
        self.tmo2 = TestModelOne.objects.get(pk=self.tmo2.pk)
        self.assertFalse(self.tmo1.deleted)
        self.assertFalse(self.tmo2.deleted)
        self.assertTrue(self.pre_undelete_called)
        self.assertTrue(self.post_undelete_called)


class M2MTests(BaseTest):
    def test_m2mdelete(self):
        t3 = TestModelThree.objects.all()[0]
        self.assertFalse(t3.deleted)
        for x in t3.tmos.all():
            self.assertFalse(x.deleted)
        t3.delete()
        for x in t3.tmos.all():
            self.assertFalse(x.deleted)


class SoftDeleteRelatedFieldLookupsTests(BaseTest):
    def test_related_foreign_key(self):
        tmt1 = TestModelTwoCascade.objects.create(extra_int=100, tmo=self.tmo1)
        tmt2 = TestModelTwoCascade.objects.create(extra_int=100, tmo=self.tmo2)

        self.assertEquals(self.tmo1.tmts.filter(extra_int=100).count(), 1)
        self.assertEquals(self.tmo1.tmts.filter(extra_int=100)[0].pk, tmt1.pk)
        self.assertEquals(self.tmo2.tmts.filter(extra_int=100).count(), 1)
        self.assertEquals(self.tmo2.tmts.filter(extra_int=100)[0].pk, tmt2.pk)

        self.assertEquals(self.tmo1.tmts.get(extra_int=100), tmt1)
        self.assertEquals(self.tmo2.tmts.get(extra_int=100), tmt2)

        tmt1.delete()
        self.assertEquals(self.tmo1.tmts.filter(extra_int=100).count(), 0)
        tmt1.undelete()
        self.assertEquals(self.tmo1.tmts.filter(extra_int=100).count(), 1)

        tmt1.delete()
        tmt1.delete()
        self.assertRaises(TestModelTwoCascade.DoesNotExist,
                          self.tmo1.tmts.get, extra_int=100)

    def test_related_m2m(self):
        t31 = TestModelThree.objects.create(extra_int=100)
        TestModelThrough.objects.create(tmo1=self.tmo1, tmo3=t31)
        t32 = TestModelThree.objects.create(extra_int=100)
        TestModelThrough.objects.create(tmo1=self.tmo2, tmo3=t32)

        self.assertEquals(self.tmo1.testmodelthree_set.filter(extra_int=100).count(), 1)
        self.assertEquals(self.tmo1.testmodelthree_set.filter(extra_int=100)[0].pk, t31.pk)
        self.assertEquals(self.tmo2.testmodelthree_set.filter(extra_int=100).count(), 1)
        self.assertEquals(self.tmo2.testmodelthree_set.filter(extra_int=100)[0].pk, t32.pk)

        self.assertEquals(self.tmo1.testmodelthree_set.get(extra_int=100), t31)
        self.assertEquals(self.tmo2.testmodelthree_set.get(extra_int=100), t32)

        t31.delete()
        self.assertEquals(self.tmo1.testmodelthree_set.filter(extra_int=100).count(), 0)
        t31.undelete()
        self.assertEquals(self.tmo1.testmodelthree_set.filter(extra_int=100).count(), 1)

        t31.delete()
        t31.delete()
        self.assertRaises(TestModelThree.DoesNotExist,
                          self.tmo1.testmodelthree_set.get, extra_int=100)

    def test_one_to_one(self):
        bob = TestModelBaseO2OMale.objects.create(name='Bob')
        alice = TestModelO2OFemaleSetNull.objects.create(name='Alice', link=bob)

        bob.delete()

        self.assertEquals(alice.link_id, None)

        romeo = TestModelBaseO2OMale.objects.create(name='Romeo')
        juliet = TestModelO2OFemaleCascade.objects.create(name='Juliet', link=romeo)

        romeo.delete()

        self.assertRaises(TestModelO2OFemaleCascade.DoesNotExist, TestModelO2OFemaleCascade.objects.get, name='Juliet')
        self.assertEquals(juliet.deleted, True)

        kurt = TestModelBaseO2OMale.objects.create(name='Kurt')
        courtney = TestModelO2OFemaleCascadeNoSD.objects.create(name='Courtney', link=kurt)
        jack = TestModelBaseO2OMale.objects.create(name='Jack')
        jill = TestModelO2OFemaleCascadeNoSD.objects.create(name='jill', link=jack)

        kurt.delete()

        self.assertTrue(TestModelO2OFemaleCascadeNoSD.objects.filter(id=jill.id).exists())
        self.assertFalse(TestModelO2OFemaleCascadeNoSD.objects.filter(id=courtney.id).exists())

    @override_settings(SOFTDELETE_CASCADE_ALLOW_DELETE_ALL=True)
    def test_fallback_delete_all_setting_false(self):
        bob = TestModelBaseO2OMale.objects.create(name='Bob')
        TestModelO2OFemaleCascadeErrorOnDelete.objects.create(name='Alice', link=bob)

        romeo = TestModelBaseO2OMale.objects.create(name='Romeo')
        TestModelO2OFemaleCascadeErrorOnDelete.objects.create(name='Juliet', link=romeo)

        self.assertFalse(TestModelO2OFemaleCascadeErrorOnDelete.objects.all().exists())

    @override_settings(SOFTDELETE_CASCADE_ALLOW_DELETE_ALL=False)
    def test_fallback_delete_all_setting_false(self):
        bob = TestModelBaseO2OMale.objects.create(name='Bob')
        alice = TestModelO2OFemaleCascadeErrorOnDelete.objects.create(name='Alice', link=bob)

        romeo = TestModelBaseO2OMale.objects.create(name='Romeo')
        TestModelO2OFemaleCascadeErrorOnDelete.objects.create(name='Juliet', link=romeo)

        self.assertRaises(ModelDeletionException, alice.delete)
        self.assertTrue(TestModelO2OFemaleCascadeErrorOnDelete.objects.filter(id=alice.id).exists())
        self.assertEquals(TestModelO2OFemaleCascadeErrorOnDelete.objects.count(), 2)



from softdelete.admin.admin import *
from softdelete.admin.forms import *

__all__ = ['SoftDeleteObjectAdmin',
           'SoftDeleteRecordAdmin',
           'ChangeSetAdmin',
           'SoftDeleteObjectInline',
           'SoftDeleteObjectAdminForm',
           ]




from softdelete.test_softdelete_app.models import TestModelTwoCascade, TestModelTwoDoNothing, TestModelTwoSetNull

TEST_MODEL_ONE_COUNT = 2
TEST_MODEL_TWO_TOTAL_COUNT = 12  # should be multiple of LCM(TEST_MODEL_ONE_COUNT,TEST_MODEL_TWO_COUNT),
# here LCM is 6 and multiplier is 2

TEST_MODEL_THREE_COUNT = TEST_MODEL_TWO_TOTAL_COUNT ** 2
TEST_MODEL_TWO_LIST = [TestModelTwoCascade,
                       TestModelTwoDoNothing,
                       TestModelTwoSetNull]
TEST_MODEL_TWO_CASCADE_COUNT = TEST_MODEL_TWO_DO_NOTHING_COUNT = TEST_MODEL_TWO_SET_NULL_COUNT = \
    TEST_MODEL_TWO_TOTAL_COUNT // len(TEST_MODEL_TWO_LIST)



from django.db import models
from django.contrib import admin
from django.db.models import QuerySet

from softdelete.models import *
from softdelete.admin import *
from softdelete.test_softdelete_app.exceptions import ModelDeletionException


class TestModelOne(SoftDeleteObject):
    extra_bool = models.BooleanField(default=False)


class TestModelTwoCascade(SoftDeleteObject):
    extra_int = models.IntegerField()
    tmo = models.ForeignKey(
        TestModelOne,
        on_delete=models.CASCADE,
        related_name='tmts'
    )


class TestModelTwoDoNothing(SoftDeleteObject):
    extra_int = models.IntegerField()
    tmo = models.ForeignKey(
        TestModelOne,
        on_delete=models.DO_NOTHING,
        related_name='tmdn'
    )


class TestModelTwoSetNull(SoftDeleteObject):
    extra_int = models.IntegerField()
    tmo = models.ForeignKey(
        TestModelOne,
        on_delete=models.SET_NULL,
        related_name='tmsn',
        null=True,
        blank=True
    )


class TestModelTwoSetNullOneToOne(SoftDeleteObject):
    extra_int = models.IntegerField()
    tmo = models.OneToOneField(
        TestModelOne,
        on_delete=models.SET_NULL,
        related_name='tmsno',
        null=True,
        blank=True
    )


class TestModelThree(SoftDeleteObject):
    tmos = models.ManyToManyField(TestModelOne, through='TestModelThrough')
    extra_int = models.IntegerField(blank=True, null=True)


class TestModelThrough(SoftDeleteObject):
    tmo1 = models.ForeignKey(
        TestModelOne,
        on_delete=models.CASCADE,
        related_name="left_side"
    )
    tmo3 = models.ForeignKey(
        TestModelThree,
        on_delete=models.CASCADE,
        related_name='right_side'
    )


class TestModelBaseO2OMale(SoftDeleteObject):
    name = models.CharField(max_length=16)


class TestModelO2OFemaleSetNull(SoftDeleteObject):
    name = models.CharField(max_length=16)
    link = models.OneToOneField(
        TestModelBaseO2OMale,
        related_name='one_to_one_set_null',
        on_delete=models.SET_NULL,
        null=True,
    )


class TestModelO2OFemaleCascade(SoftDeleteObject):
    name = models.CharField(max_length=16)
    link = models.OneToOneField(
        TestModelBaseO2OMale,
        related_name='one_to_one_cascade',
        on_delete=models.CASCADE,
    )


class TestModelO2OFemaleCascadeErrorOnDelete(models.Model):
    name = models.CharField(max_length=16)
    link = models.OneToOneField(
        TestModelBaseO2OMale,
        related_name='one_to_one_cascade_error_on_delete',
        on_delete=models.CASCADE,
    )

    def delete(self, using=None, keep_parents=False):
        raise ModelDeletionException("Preventing deletion!")


class TestModelO2OFemaleCascadeNoSD(models.Model):
    name = models.CharField(max_length=16)
    link = models.OneToOneField(
        TestModelBaseO2OMale,
        related_name='one_to_one_cascade_no_sd',
        on_delete=models.CASCADE
    )


admin.site.register(TestModelOne, SoftDeleteObjectAdmin)
admin.site.register(TestModelTwoCascade, SoftDeleteObjectAdmin)
admin.site.register(TestModelTwoSetNull, SoftDeleteObjectAdmin)
admin.site.register(TestModelTwoDoNothing, SoftDeleteObjectAdmin)

admin.site.register(TestModelThree, SoftDeleteObjectAdmin)
admin.site.register(TestModelThrough, SoftDeleteObjectAdmin)



from django.forms import *
from softdelete.models import *
from softdelete.forms import *
import logging

class SoftDeleteObjectAdminForm(ModelForm):
    deleted = BooleanField(required=False)
    
    class Meta:
        model = SoftDeleteObject
        exclude = ('deleted_at',)

    def __init__(self, *args, **kwargs):
        super(SoftDeleteObjectAdminForm, self).__init__(*args, **kwargs)
        instance = kwargs.get('instance')
        if instance:
            self.initial['deleted'] = instance.deleted

    def clean(self, *args, **kwargs):
        cleaned_data = super(SoftDeleteObjectAdminForm, self).clean(*args, **kwargs)
        if 'undelete' in self.data:
            self.instance.deleted = False
            cleaned_data['deleted'] = False
        return cleaned_data

    def save(self, commit=True, *args, **kwargs):
        model = super(SoftDeleteObjectAdminForm, self).save(commit=False, 
                                                            *args, **kwargs)
        model.deleted = self.cleaned_data['deleted']
        if commit:
            model.save()
        return model

class ChangeSetAdminForm(ChangeSetForm):
    pass

class SoftDeleteRecordAdminForm(ModelForm):
    class Meta:
        model = SoftDeleteRecord
        readonly_fields = ('created', )
        exclude = ('content_type', 'object_id', 'changeset')




import logging

from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.conf import settings
from django.shortcuts import get_object_or_404, redirect
from django.core import serializers
from django.contrib import auth
from django.contrib.auth.decorators import permission_required
from django.forms.models import inlineformset_factory
from django.views.generic import *
from django.views.generic.base import TemplateResponseMixin, View
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from django.template import RequestContext
try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from softdelete.forms import *
from softdelete.models import *


class ProtectedView(object):
    @method_decorator(permission_required('softdelete.can_undelete'))
    def dispatch(self, *args, **kwargs):
        return super(ProtectedView, self).dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ProtectedView, self).get_context_data(**kwargs)
        context['request'] = self.request
        return context

class ChangeSetList(ProtectedView, ListView):
    model = ChangeSet

    def get_query_set(self):
        return self.model.objects.all()

    def get_queryset(self):
        return self.model.objects.all()

class ChangeSetDetail(ProtectedView, DetailView):
    model = ChangeSet

    def get_object(self):
        return get_object_or_404(ChangeSet, pk=self.kwargs['changeset_pk'])

class ChangeSetUpdate(ProtectedView, UpdateView):
    model = ChangeSet
    form_class = ChangeSetForm

    def get_object(self):
        return get_object_or_404(ChangeSet, pk=self.kwargs['changeset_pk'])

    def get_success_url(self):
        return reverse('softdelete.changeset.list')

    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super(ChangeSetUpdate, self).dispatch(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        if request.POST.get('action') != 'Undelete':
            return HttpResponseRedirect(reverse('softdelete.changeset.view',
                                                args=(kwargs['changeset_pk'],)))
        self.get_object().undelete()
        return HttpResponseRedirect(self.get_success_url())



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'SoftDeleteRecord.object_id'
        db.alter_column(u'softdelete_softdeleterecord', 'object_id', self.gf('django.db.models.fields.CharField')(max_length=100))

        # Changing field 'ChangeSet.object_id'
        db.alter_column(u'softdelete_changeset', 'object_id', self.gf('django.db.models.fields.CharField')(max_length=100))

    def backwards(self, orm):

        # Changing field 'SoftDeleteRecord.object_id'
        db.alter_column(u'softdelete_softdeleterecord', 'object_id', self.gf('django.db.models.fields.PositiveIntegerField')())

        # Changing field 'ChangeSet.object_id'
        db.alter_column(u'softdelete_changeset', 'object_id', self.gf('django.db.models.fields.PositiveIntegerField')())

    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'softdelete.changeset': {
            'Meta': {'object_name': 'ChangeSet'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'created_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.utcnow'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'softdelete.softdeleterecord': {
            'Meta': {'unique_together': "(('changeset', 'content_type', 'object_id'),)", 'object_name': 'SoftDeleteRecord'},
            'changeset': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'soft_delete_records'", 'to': u"orm['softdelete.ChangeSet']"}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'created_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.utcnow'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['softdelete']


from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.core.exceptions import ObjectDoesNotExist
from django.contrib import admin
from django.db import models
from softdelete.models import *
from softdelete.admin.forms import *
import logging


class SoftDeleteObjectInline(admin.TabularInline):
    class Meta:
        exclude = ('deleted_at',)

    def __init__(self, parent, site, *args, **kwargs):
        super(SoftDeleteObjectInline, self).__init__(parent, site, *args, **kwargs)
        if parent.deleted:
            self.extra = 0
            self.max_num = 0

    def get_queryset(self, request):
        qs = self.model._default_manager.all_with_deleted()
        ordering = self.get_ordering(request) or ()
        if ordering:
            qs = qs.order_by(*ordering)
        return qs

    queryset = get_queryset


class SoftDeleteObjectAdmin(admin.ModelAdmin):
    form = SoftDeleteObjectAdminForm
    actions = ['delete_selected', 'soft_undelete', 'hard_delete']

    def delete_selected(self, request, queryset):
        queryset.delete()

    delete_selected.short_description = 'Soft delete selected objects'

    def soft_undelete(self, request, queryset):
        queryset.undelete()

    soft_undelete.short_description = 'Undelete selected objects'

    def hard_delete(self, request, queryset):
        for obj in queryset:
            if obj.deleted_at:
                obj.delete()
            else:
                obj.hard_delete()
    hard_delete.short_description = 'Hard delete selected objects'

    def response_change(self, request, obj, *args, **kwargs):
        if 'undelete' in request.POST:
            return HttpResponseRedirect('../')
        return super(SoftDeleteObjectAdmin, self).response_change(request, obj, *args, **kwargs)

    def get_queryset(self, request):
        try:
            qs = self.model._default_manager.all_with_deleted()
        except Exception as ex:
            qs = self.model._default_manager.all()

        ordering = self.get_ordering(request) or ()
        if ordering:
            qs = qs.order_by(*ordering)
        return qs

    queryset = get_queryset


class SoftDeleteRecordInline(admin.TabularInline):
    model = SoftDeleteRecord
    max_num = 0
    exclude = ('content_type', 'object_id',)
    readonly_fields = ('content',)


class SoftDeleteRecordAdmin(admin.ModelAdmin):
    model = SoftDeleteRecord
    form = SoftDeleteRecordAdminForm
    actions = ['soft_undelete']

    def soft_undelete(self, request, queryset):
        [x.undelete() for x in queryset.all()]

    soft_undelete.short_description = 'Undelete selected objects'

    def response_change(self, request, obj, *args, **kwargs):
        if 'undelete' in request.POST:
            obj.undelete()
            return HttpResponseRedirect('../../')
        return super(SoftDeleteRecordAdmin, self).response_change(request, obj,
                                                                  *args, **kwargs)

    def has_add_permission(self, *args, **kwargs):
        return False

    def has_delete_permission(self, *args, **kwargs):
        return False


class ChangeSetAdmin(admin.ModelAdmin):
    model = ChangeSet
    form = ChangeSetAdminForm
    inlines = (SoftDeleteRecordInline,)
    actions = ['soft_undelete']

    def soft_undelete(self, request, queryset):
        [x.undelete() for x in queryset.all()]

    soft_undelete.short_description = 'Undelete selected objects'

    def response_change(self, request, obj, *args, **kwargs):
        if 'undelete' in request.POST:
            obj.undelete()
            return HttpResponseRedirect('../../')
        return super(ChangeSetAdmin, self).response_change(request, obj,
                                                           *args, **kwargs)

    def has_add_permission(self, *args, **kwargs):
        return False

    def has_delete_permission(self, *args, **kwargs):
        return False


admin.site.register(SoftDeleteRecord, SoftDeleteRecordAdmin)
admin.site.register(ChangeSet, ChangeSetAdmin)



import sys
import os
import django

BASE_DIR = os.path.dirname(__file__)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'my_db',
        }
    }

INSTALLED_APPS = [
    'softdelete',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.admin',
    'django.contrib.messages',
]

if django.VERSION[0] >= 2:
    MIDDLEWARE = [
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
    ]
    SILENCED_SYSTEM_CHECKS = (
        'admin.E130',
    )
else:
    MIDDLEWARE_CLASSES= [
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
    ]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(BASE_DIR, 'tests/templates')
        ],
        'OPTIONS': {
             'debug': True,
             'loaders': (
                  'django.template.loaders.filesystem.Loader',
                  'django.template.loaders.app_directories.Loader',
              ),
             'context_processors': (
                 'django.contrib.messages.context_processors.messages',
                 'django.contrib.auth.context_processors.auth',
                 'django.template.context_processors.request'
             )
         }
    },
]


DOMAIN = 'http://testserver'
ROOT_URLCONF = 'softdelete.urls'
SECRET_KEY = "dummy"
DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

if 'test' in sys.argv:
    INSTALLED_APPS.append("softdelete.test_softdelete_app")



from django.conf import settings
from django.test import TestCase, Client
from django.contrib.auth.models import User
from django.db import models
from softdelete.test_softdelete_app.models import TestModelOne, TestModelTwoCascade, TestModelThree
from softdelete.models import *
from softdelete.signals import *
import logging
try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse


class ViewBase(TestCase):
    def setUp(self):
        u, c = User.objects.get_or_create(username="undelete_test")
        u.is_active = True
        u.set_password("undelete_password")
        gr = create_group()
        if USE_SOFTDELETE_GROUP:
            gr = Group.objects.get(name="Softdelete User")
            u.groups.add(gr)
            u.save()
            gr.save()
        else:
            assign_permissions(u)
        u.save()
        self.tmo1 = TestModelOne.objects.create(extra_bool=True)
        self.tmo3 = TestModelThree.objects.create(extra_int=3)
        for x in range(10):
            TestModelTwoCascade.objects.create(extra_int=x, tmo=self.tmo1)
        self.tmo2 = TestModelOne.objects.create(extra_bool=False)
        for x in range(10):
            TestModelTwoCascade.objects.create(extra_int=x * x, tmo=self.tmo2)
        self.tmo2.delete()


class ViewTest(ViewBase):
    def __init__(self, *args, **kwargs):
        settings.USE_SOFTDELETE_GROUP = kwargs.get('USE_SOFTDELETE_GROUP', False)
        if 'USE_SOFTDELETE_GROUP' in kwargs:
            del kwargs['USE_SOFTDELETE_GROUP']
        super(ViewTest, self).__init__(*args, **kwargs)

    def setUp(self):
        super(ViewTest, self).setUp()
        self.client = Client()
        self.client.login(username="undelete_test",
                          password="undelete_password")

    def test_authorization(self):
        rv = self.client.get(reverse("softdelete.changeset.list"))
        pk = ChangeSet.objects.latest('created_date').pk
        for view_name in [reverse("softdelete.changeset.list"),
                          reverse("softdelete.changeset.view", args=(pk,)),
                          reverse("softdelete.changeset.undelete", args=(pk,)),]:
            cli2 = Client()
            rv = cli2.get(view_name)
            # Make sure we redirected to a login page
            self.assertEquals(rv.status_code, 302)
            self.assertIn(reverse('login'), rv['Location'])
            # But don't try to render it.

    def test_undelete(self):
        self.cs_count = ChangeSet.objects.count()
        self.rs_count = SoftDeleteRecord.objects.count()
        self.t_count = TestModelThree.objects.count()
        self.tmo3.delete()
        self.assertEquals(self.t_count-1, TestModelThree.objects.count())
        self.assertEquals(0, self.tmo3.tmos.count())
        self.assertEquals(self.cs_count+1, ChangeSet.objects.count())
        self.assertEquals(self.rs_count+1, SoftDeleteRecord.objects.count())
        rv = self.client.get(reverse("softdelete.changeset.undelete",
                                     args=(ChangeSet.objects.latest("created_date").pk,)))
        self.assertEquals(rv.status_code,200)
        rv = self.client.post(reverse("softdelete.changeset.undelete",
                                     args=(ChangeSet.objects.latest("created_date").pk,)),
                             {'action': 'Undelete'})
        self.assertEquals(rv.status_code, 302)
        rv = self.client.get(rv['Location'])
        self.assertEquals(rv.status_code, 200)
        self.assertEquals(self.cs_count, ChangeSet.objects.count())
        self.assertEquals(self.rs_count, SoftDeleteRecord.objects.count())
        self.assertEquals(self.t_count, TestModelThree.objects.count())
        self.assertEquals(0, self.tmo3.tmos.count())


class GroupViewTest(ViewTest):
    def __init__(self, *args, **kwargs):
        super(GroupViewTest, self).__init__(USE_SOFTDELETE_GROUP=True, *args, **kwargs)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'ChangeSet'
        db.create_table('softdelete_changeset', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created_date', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.utcnow)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal('softdelete', ['ChangeSet'])

        # Adding model 'RecordSet'
        db.create_table('softdelete_recordset', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('changeset', self.gf('django.db.models.fields.related.ForeignKey')(related_name='recordsets', to=orm['softdelete.ChangeSet'])),
            ('created_date', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.utcnow)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal('softdelete', ['RecordSet'])

        # Adding unique constraint on 'RecordSet', fields ['changeset', 'content_type', 'object_id']
        db.create_unique('softdelete_recordset', ['changeset_id', 'content_type_id', 'object_id'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'RecordSet', fields ['changeset', 'content_type', 'object_id']
        db.delete_unique('softdelete_recordset', ['changeset_id', 'content_type_id', 'object_id'])

        # Deleting model 'ChangeSet'
        db.delete_table('softdelete_changeset')

        # Deleting model 'RecordSet'
        db.delete_table('softdelete_recordset')


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'softdelete.changeset': {
            'Meta': {'object_name': 'ChangeSet'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'created_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.utcnow'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        'softdelete.recordset': {
            'Meta': {'unique_together': "(('changeset', 'content_type', 'object_id'),)", 'object_name': 'RecordSet'},
            'changeset': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'recordsets'", 'to': "orm['softdelete.ChangeSet']"}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'created_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.utcnow'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {})
        }
    }

    complete_apps = ['softdelete']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ChangeSet',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('object_id', models.CharField(max_length=100)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
        ),
        migrations.CreateModel(
            name='SoftDeleteRecord',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('object_id', models.CharField(max_length=100)),
                ('changeset', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='soft_delete_records', to='softdelete.ChangeSet')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='softdeleterecord',
            unique_together=set([('changeset', 'content_type', 'object_id')]),
        ),
    ]



from setuptools import setup, find_packages

setup(name='django-softdelete-ts',
      version='0.9.5',
      description='Soft delete support for Django ORM, with undelete.',
      author='Steve Coursen',
      author_email='smcoursen@gmail.com',
      maintainer='Steve Coursen',
      maintainer_email='smcoursen@gmail.com',
      license="BSD",
      zip_safe=True,
      url="https://github.com/jackeygao/django-softdelete",
      packages=find_packages(),
      install_requires=['setuptools','wheel','six'],
      include_package_data=True,
      setup_requires=[],
      classifiers=[
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
        'Environment :: Web Environment',
        ]
)
